@@ -98,11 +98,11 @@ void mainchan_terminal_size(mainchan *mc, int width, int height) {}
 /* Seat functions to ensure we don't get choosy about crypto - as the
  * server, it's not up to us to give user warnings */
 static SeatPromptResult server_confirm_weak_crypto_primitive(
-    Seat *seat, const char *algtype, const char *algname,
+    Seat *seat, SeatDialogText *text,
     void (*callback)(void *ctx, SeatPromptResult result), void *ctx)
 { return SPR_OK; }
 static SeatPromptResult server_confirm_weak_cached_hostkey(
-    Seat *seat, const char *algname, const char *betteralgs,
+    Seat *seat, SeatDialogText *text,
     void (*callback)(void *ctx, SeatPromptResult result), void *ctx)
 { return SPR_OK; }
 
