@@ -32,20 +32,18 @@ void console_print_error_msg(const char *prefix, const char *msg)
     fflush(stderr);
 }
 
-SeatPromptResult console_confirm_ssh_host_key(
-    Seat *seat, const char *host, int port, const char *keytype,
-    char *keystr, SeatDialogText *text, HelpCtx helpctx,
-    void (*callback)(void *ctx, SeatPromptResult result), void *ctx)
+/*
+ * Helper function to print the message from a SeatDialogText. Returns
+ * the final prompt to print on the input line, or NULL if a
+ * batch-mode abort is needed. In the latter case it will have printed
+ * the abort text already.
+ */
+static const char *console_print_seatdialogtext(SeatDialogText *text)
 {
-    HANDLE hin;
-    DWORD savemode, i;
     const char *prompt = NULL;
-
     stdio_sink errsink[1];
     stdio_sink_init(errsink, stderr);
 
-    char line[32];
-
     for (SeatDialogTextItem *item = text->items,
              *end = item+text->nitems; item < end; item++) {
         switch (item->type) {
@@ -65,7 +63,7 @@ SeatPromptResult console_confirm_ssh_host_key(
             if (console_batch_mode) {
                 fprintf(stderr, "%s\n", item->text);
                 fflush(stderr);
-                return SPR_SW_ABORT("Cannot confirm a host key in batch mode");
+                return NULL;
             }
             break;
           case SDT_PROMPT:
@@ -76,6 +74,22 @@ SeatPromptResult console_confirm_ssh_host_key(
         }
     }
     assert(prompt); /* something in the SeatDialogText should have set this */
+    return prompt;
+}
+
+SeatPromptResult console_confirm_ssh_host_key(
+    Seat *seat, const char *host, int port, const char *keytype,
+    char *keystr, SeatDialogText *text, HelpCtx helpctx,
+    void (*callback)(void *ctx, SeatPromptResult result), void *ctx)
+{
+    HANDLE hin;
+    DWORD savemode, i;
+
+    const char *prompt = console_print_seatdialogtext(text);
+    if (!prompt)
+        return SPR_SW_ABORT("Cannot confirm a host key in batch mode");
+
+    char line[32];
 
     while (true) {
         fprintf(stderr,
@@ -128,23 +142,20 @@ SeatPromptResult console_confirm_ssh_host_key(
 }
 
 SeatPromptResult console_confirm_weak_crypto_primitive(
-    Seat *seat, const char *algtype, const char *algname,
+    Seat *seat, SeatDialogText *text,
     void (*callback)(void *ctx, SeatPromptResult result), void *ctx)
 {
     HANDLE hin;
     DWORD savemode, i;
 
-    char line[32];
-
-    fprintf(stderr, weakcrypto_msg_common_fmt, algtype, algname);
-
-    if (console_batch_mode) {
-        fputs(console_abandoned_msg, stderr);
+    const char *prompt = console_print_seatdialogtext(text);
+    if (!prompt)
         return SPR_SW_ABORT("Cannot confirm a weak crypto primitive "
                             "in batch mode");
-    }
 
-    fputs(console_continue_prompt, stderr);
+    char line[32];
+
+    fprintf(stderr, "%s (y/n) ", prompt);
     fflush(stderr);
 
     hin = GetStdHandle(STD_INPUT_HANDLE);
@@ -163,23 +174,20 @@ SeatPromptResult console_confirm_weak_crypto_primitive(
 }
 
 SeatPromptResult console_confirm_weak_cached_hostkey(
-    Seat *seat, const char *algname, const char *betteralgs,
+    Seat *seat, SeatDialogText *text,
     void (*callback)(void *ctx, SeatPromptResult result), void *ctx)
 {
     HANDLE hin;
     DWORD savemode, i;
 
-    char line[32];
-
-    fprintf(stderr, weakhk_msg_common_fmt, algname, betteralgs);
-
-    if (console_batch_mode) {
-        fputs(console_abandoned_msg, stderr);
+    const char *prompt = console_print_seatdialogtext(text);
+    if (!prompt)
         return SPR_SW_ABORT("Cannot confirm a weak cached host key "
                             "in batch mode");
-    }
 
-    fputs(console_continue_prompt, stderr);
+    char line[32];
+
+    fprintf(stderr, "%s (y/n) ", prompt);
     fflush(stderr);
 
     hin = GetStdHandle(STD_INPUT_HANDLE);
